//
// Copyright (c) 2019 Vinnie Falco (vinnie.falco@gmail.com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//
// Official repository: https://github.com/boostorg/json
//

#ifndef BOOST_JSON_FWD_HPP
#define BOOST_JSON_FWD_HPP

#include <boost/json/detail/config.hpp>

BOOST_JSON_NS_BEGIN

// Forward declarations for value types

#ifndef BOOST_JSON_DOCS

class array;
class object;
class string;
class value;

class key_value_pair;
class storage_ptr;

#endif

BOOST_JSON_NS_END

#endif
